@@ -1,5 +1,5 @@
 from dataclasses import dataclass
-from typing import Union, Optional, Callable
+from typing import Union, Optional, Callable, List, Tuple
 
 import torch
 from transformers import CLIPTokenizer, CLIPTextModel
@@ -21,11 +21,13 @@ class Compel:
                  tokenizer: CLIPTokenizer,
                  text_encoder: CLIPTextModel,
                  textual_inversion_manager: Optional[BaseTextualInversionManager] = None,
-                 dtype_for_device_getter: Callable[[torch.device], torch.dtype] = lambda device: torch.float32):
+                 dtype_for_device_getter: Callable[[torch.device], torch.dtype] = lambda device: torch.float32,
+                 truncate_long_prompts: bool=True):
         self.conditioning_provider = EmbeddingsProvider(tokenizer=tokenizer,
                                                         text_encoder=text_encoder,
                                                         textual_inversion_manager=textual_inversion_manager,
-                                                        dtype_for_device_getter=dtype_for_device_getter
+                                                        dtype_for_device_getter=dtype_for_device_getter,
+                                                        truncate=truncate_long_prompts
                                                         )
 
     @property
@@ -35,6 +37,7 @@ class Compel:
     def make_conditioning_scheduler(self, positive_prompt: str, negative_prompt: str='') -> ConditioningScheduler:
         positive_conditioning = self.build_conditioning_tensor(positive_prompt)
         negative_conditioning = self.build_conditioning_tensor(negative_prompt)
+        positive_conditioning, negative_conditioning = self.pad_conditioning_tensors_to_same_length(positive_conditioning, negative_conditioning)
         return StaticConditioningScheduler(positive_conditioning=positive_conditioning,
                                            negative_conditioning=negative_conditioning)
 
@@ -43,6 +46,19 @@ class Compel:
         conditioning, _ = self.build_conditioning_tensor_for_prompt_object(prompt_object)
         return conditioning
 
+    @torch.no_grad()
+    def __call__(self, text: Union[str, List[str]]) -> torch.Tensor:
+        if not isinstance(text, list):
+            text = [text]
+
+        cond_tensor = []
+        for text_input in text:
+            cond_tensor.append(self.build_conditioning_tensor(text_input))
+
+        cond_tensor = torch.cat(cond_tensor)
+
+        return cond_tensor
+
     @classmethod
     def parse_prompt_string(cls, prompt_string: str) -> Union[FlattenedPrompt, Blend]:
         pp = PromptParser()
@@ -51,7 +67,7 @@ class Compel:
         parsed_prompt = conjunction.prompts[0]
         return parsed_prompt
 
-    def describe_tokenization(self, text: str) -> list[str]:
+    def describe_tokenization(self, text: str) -> List[str]:
         """
         For the given text, return a list of strings showing how it will be tokenized.
 
@@ -63,7 +79,7 @@ class Compel:
         return self.conditioning_provider.tokenizer.tokenize(text)
 
     def build_conditioning_tensor_for_prompt_object(self, prompt: Union[Blend, FlattenedPrompt],
-                                                    ) -> tuple[torch.Tensor, dict]:
+                                                    ) -> Tuple[torch.Tensor, dict]:
         """
 
         """
@@ -78,8 +94,28 @@ class Compel:
 
         raise ValueError(f"unsupported prompt type: {type(prompt).__name__}")
 
+    def pad_conditioning_tensors_to_same_length(self, c0: torch.Tensor, c1: torch.Tensor
+                                                ) -> Tuple[torch.Tensor, torch.Tensor]:
+        """
+        If `truncate_long_prompts` was set to False on initialization, conditioning tensors do not have a fixed length.
+        This is a problem when using a negative and a positive prompt to condition the diffusion process. This function
+        pads either c0 or c1 if necessary to ensure they both have the same length, returning the padded c0 and c1.
+        """
+        if c0.shape == c1.shape:
+            return c0, c1
+
+        max_conditioning_tensor_length = max(c0.shape[1], c1.shape[1])
+        # pad out with an emptystring tensor
+        empty_z = self.build_conditioning_tensor("")
+        while c0.shape[1] < max_conditioning_tensor_length:
+            c0 = torch.cat([c0, empty_z], dim=1)
+        while c1.shape[1] < max_conditioning_tensor_length:
+            c1 = torch.cat([c1, empty_z], dim=1)
+        return c0, c1
+
+
     def _get_conditioning_for_flattened_prompt(self, prompt: FlattenedPrompt, should_return_tokens: bool=False
-                                               ) -> Union[torch.Tensor, tuple[torch.Tensor, torch.Tensor]]:
+                                               ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         if type(prompt) is not FlattenedPrompt:
             raise ValueError(f"embeddings can only be made from FlattenedPrompts, got {type(prompt).__name__} instead")
         fragments = [x.text for x in prompt.children]
@@ -175,6 +211,6 @@ class Compel:
         tokens = self.conditioning_provider.get_token_ids(texts, include_start_and_end_markers=False)
         return sum([len(x) for x in tokens])
 
-    def get_tokens(self, text: str) -> list[int]:
+    def get_tokens(self, text: str) -> List[int]:
         return self.conditioning_provider.get_token_ids([text], include_start_and_end_markers=False)[0]
 
